@@ -13,6 +13,13 @@ from typing import Any, Dict, List, Optional
 import httpx
 import websockets
 
+from .safety import (
+    safety_filter,
+    is_safe_tool_call,
+    create_safety_response,
+    sanitize_tool_call,
+)
+
 
 class TransportProtocol(ABC):
     """Abstract base class for transport protocols."""
@@ -53,8 +60,33 @@ class TransportProtocol(ABC):
 
     async def call_tool(self, tool_name: str, arguments: Dict[str, Any]) -> Any:
         """Call a specific tool with arguments."""
-        params = {"name": tool_name, "arguments": arguments}
-        return await self.send_request("tools/call", params)
+        # Apply safety filtering
+        if not is_safe_tool_call(tool_name, arguments):
+            safety_filter.log_blocked_operation(
+                tool_name, arguments, "Dangerous tool call blocked in transport"
+            )
+            return create_safety_response(tool_name)
+
+        # Sanitize arguments if needed (even if not completely blocked)
+        sanitized_tool_name, sanitized_arguments = sanitize_tool_call(
+            tool_name, arguments
+        )
+
+        # Check if arguments were sanitized
+        safety_sanitized = sanitized_arguments != arguments
+
+        params = {"name": sanitized_tool_name, "arguments": sanitized_arguments}
+        result = await self.send_request("tools/call", params)
+
+        # Add safety information to the result if it was sanitized
+        if safety_sanitized and isinstance(result, dict):
+            if "_meta" not in result:
+                result["_meta"] = {}
+            result["_meta"]["safety_sanitized"] = True
+            result["_meta"]["original_arguments"] = arguments
+            result["_meta"]["sanitized_arguments"] = sanitized_arguments
+
+        return result
 
 
 class HTTPTransport(TransportProtocol):
@@ -114,6 +146,7 @@ class HTTPTransport(TransportProtocol):
                 logging.error("Server returned error: %s", data["error"])
                 raise Exception(f"Server error: {data['error']}")
 
+            # Return the result if it exists, otherwise return the full data
             return data.get("result", data)
 
 
@@ -210,7 +243,38 @@ class StdioTransport(TransportProtocol):
         logging.debug("Process stderr: %s", stderr_text)
 
         try:
-            data = json.loads(stdout_text)
+            # Handle multiple JSON objects in the response
+            # Split by newlines and try to parse each line as JSON
+            lines = stdout_text.strip().split("\n")
+            main_response = None
+
+            for line in lines:
+                line = line.strip()
+                if not line:
+                    continue
+
+                try:
+                    json_obj = json.loads(line)
+
+                    # Skip notification messages
+                    if json_obj.get("method") == "notifications/message":
+                        continue
+
+                    # Look for the main response (has "result" or "error" field)
+                    if "result" in json_obj or "error" in json_obj:
+                        main_response = json_obj
+                        break
+
+                except json.JSONDecodeError:
+                    # Skip lines that aren't valid JSON
+                    continue
+
+            if main_response is None:
+                # If no main response found, raise JSONDecodeError
+                raise json.JSONDecodeError("No main response found", stdout_text, 0)
+            else:
+                data = main_response
+
             if "error" in data:
                 logging.error("Server returned error: %s", data["error"])
                 raise Exception(f"Server error: {data['error']}")
@@ -218,7 +282,7 @@ class StdioTransport(TransportProtocol):
         except json.JSONDecodeError:
             logging.error("Failed to parse response as JSON: %s", stdout_text)
             logging.error("Process stderr: %s", stderr_text)
-            raise Exception("Invalid JSON response")
+            raise
 
 
 class WebSocketTransport(TransportProtocol):
